/*



*/

/**
   This program analyzes an Abstract Syntax Tree (AST)
   that is defined by the following grammar.

       Prog ::= Exp
              | '(' 'prog' (Fun | Exp)* Exp ')'

        Fun ::= '(' 'fun' VARIABLE Lambda ')'   // a function declaration

     Lambda ::= '(' 'lambda' VARIABLE* Exp ')'  // formal parameters followed by function body

        Exp ::= Apply
              | If
              | While
              | Var
              | Set
              | Begin
              | Print
              | BExp
              | AExp
              | INTEGER
              | BOOLEAN
              | VARIABLE

      Apply ::= '(' 'apply' Exp Exp* ')'  // function value followed by actual parameters

         If ::= '(' 'if' Exp Exp Exp ')'

      While ::= '(' 'while' Exp Exp ')'

        Var ::= '(' 'var' VARIABLE Exp ')'

        Set ::= '(' 'set' VARIABLE Exp ')'

      Begin ::= '(' 'begin' Exp* Exp ')'

      Print ::= '(' 'print' Exp ')'

       BExp ::= '(' '||'  Exp Exp+ ')'
              | '(' '&&'  Exp Exp+ ')'
              | '(' '!'   Exp ')'
              | '(' RelOp Exp Exp ')'
              | '('  EqOp Exp Exp ')'

      RelOp ::= '<' | '>' | '<=' | '>='
       EqOp ::= '==' | '!='

       AExp ::= '(' '+' Exp Exp* ')'
              | '(' '-' Exp Exp? ')'
              | '(' '*' Exp Exp+ ')'
              | '(' '/' Exp Exp  ')'
              | '(' '%' Exp Exp  ')'
              | '(' '^' Exp Exp  ')'

    INTEGER ::= [-][0-9]+
    BOOLEAN ::= 'true' | 'false'
   VARIABLE ::= [a-zA-Z][a-zA-Z0-9]*
*/

public class StaticAnalysis
{
   public static int DEBUG = 1;

   public static Environment globalEnv; // reference to the global environment object

   public static boolean analyze(Tree tree)
   {
      return analyzeProg( tree );
   }//analyze()


   // Analyze a prog.
   private static boolean analyzeProg(Tree tree)
   {
      boolean result = true;

      // Instantiate the global environment
      // (it will always be at the end of the
      // environment chain).
      globalEnv = new Environment(null, "Global Analysis");

      // Check which kind of Prog we have.
      if ( ! tree.getElement().lexeme.equals("prog") )
      {
         // Analyze the single expression.
         result = analyzeExp(tree, globalEnv);
      }
      else
      {
         boolean rv = true;
         // Analyze each Fun and gather all the function
         // names in the global environment. Do this before
         // analyzing each function's lambda expression so
         // that we can allow function forward references
         // (and mutually recursive functions).
         for (int i = 0; i < tree.degree(); i++)
         {
            if ( tree.getSubTree(i).getElement().lexeme.equals("fun") )
            {
               rv = analyzeFun_1(tree.getSubTree(i));
            }
            result = result && rv;
         }
         // Analyze each Fun or Exp in the Prog. In this pass,
         // we analyze the body (lambda expression) of each
         // function definition.
         for (int i = 0; i < tree.degree(); i++)
         {
            if ( tree.getSubTree(i).getElement().lexeme.equals("fun") )
            {
               rv = analyzeFun_2(tree.getSubTree(i));
            }
            else
            {
               rv = analyzeExp(tree.getSubTree(i), globalEnv);
            }
            result = result && rv;
         }
      }

      return result;
   }//analyzeProg()


   // Analyze the first part of a function definition.
   // This is needed by the REPL, so we make it public.
   public static boolean analyzeFun_1(Tree tree)
   {
      boolean result = true;

      // Get the function name.
      String name = tree.getSubTree(0).getElement().lexeme;
      // Check if this function has already been defined.
      if ( globalEnv.defined(name) )
      {
         result = false;
         System.err.println("**** Analysis Error: function already defined: "
                            + name
                            + " at Line " + tree.getSubTree(0).getElement().line
                            + ", Character " + tree.getSubTree(0).getElement().position
                            + "\n" );
      }
      else
      {
         // Add a <name, lambda> pair to the global Environment object.
         globalEnv.add(name, new Value(tree.getSubTree(1)));
      }

      return result;
   }//analyzeFun_1()


   // Analyze the second part of a function definition.
   // This is needed by the REPL, so we make it public.
   public static boolean analyzeFun_2(Tree tree)
   {
      return analyzeLambda(tree.getSubTree(1));
   }//analyzeFun_2()


   // Analyze a lambda expression.
   private static boolean analyzeLambda(Tree tree)
   {
      boolean result = true;




//      if (DEBUG > 0) System.out.println( tempEnv + "\n" ); // for debugging purposes

      return result;
   }//analyzeLambda()


   // Analyze an expression.
   public static boolean analyzeExp(Tree tree, Environment env)
   {
      boolean result = true;

      String node = tree.getElement().lexeme;

      if ( node.equals("var") )
         result = analyzeVar(tree, env);
      else if ( node.equals("set") )
         result = analyzeSet(tree, env);
    //else if ....




      return result;
   }//analyzeExp()


   // Analyze a function application.
   private static boolean analyzeApply(Tree tree, Environment env)
   {
      boolean result = true;



      return result;
   }//analyzeApply()


   // Analyze an if expression.
   private static boolean analyzeIf(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeIf()


   // Analyze a while expression.
   private static boolean analyzeWhile(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeWhile()


   // Analyze a variable declaration expression.
   private static boolean analyzeVar(Tree tree, Environment env)
   {
      boolean result = true;




      if (DEBUG > 0) System.out.println( env + "\n" ); // for debugging purposes

      return result;
   }//analyzeVar()


   // Analyze a variable assignment expression.
   private static boolean analyzeSet(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeSet()


   // Analyze a begin (block) expression.
   private static boolean analyzeBegin(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeBegin()


   // Analyze a print expression.
   private static boolean analyzePrint(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzePrint()


   // Analyze a boolean expression.
   private static boolean analyzeBExp(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeBExp()


   // Analyze a relational expression.
   private static boolean analyzeRExp(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeRExp()


   // Analyze an arithmetic expression.
   private static boolean analyzeAExp(Tree tree, Environment env)
   {
      boolean result = true;


      return result;
   }//analyzeAExp()

}//StaticAnalysis
