/*
   Use this program to test your parser's ability
   to catch parsing errors.

   Every Language 7 string in this file should
   produce a parsing error.
*/

public class Test_Parser_Errors
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "",
         "10 y",
         "(",
         "( )",
         "(var",
         "(var)",
         "(var (",
         "(var 5",
         "(var if",
         "(var x",
         "(var x)",
         "(var x 5",
         "(set",
         "(set)",
         "(set (",
         "(set 5",
         "(set if",
         "(set x",
         "(set x)",
         "(set x 5",
         // test prog
         "(prog (",
         "(prog ( )",
         "(prog true",
         "(prog\n(var",
         "(prog\n(var)",
         "(prog\n(var (",
         "(prog\n(var 5",
         "(prog\n(var x",
         "(prog\n(var x)",
         "(prog\n(var x 5",
         "(prog\n(set",
         "(prog\n(set)",
         "(prog\n(set (",
         "(prog\n(set 5",
         "(prog\n(set x",
         "(prog\n(set x)",
         "(prog\n(set x 5",
         "(prog\n(fun f (lambda x x)))",
         "(prog\ntrue\n(fun f (lambda x x)))",
         "(fun f (lambda x x))",
         // test if, while begin, print, apply
         "(if",
         "(if)",
         "(if (",
         "(if ( )",
         "(if true",
         "(if true)",
         "(if true (",
         "(if x y",
         "(if x y (",
         "(if x y)",
         "(if x y z",
         "(if x 10 100",
         "(if x 10 100 (",
         "(if x 10 100 1000",
         "(if x 10 100 1000)",
         "(while",
         "(while)",
         "(while true",
         "(while true (",
         "(while true x",
         "(while true x (",
         "(while true x y",
         "(while true x y)",
         "(begin",
         "(begin)",
         "(begin (",
         "(begin x",
         "(begin x (",
         "(begin x y",
         "(begin x y (",
         "(begin x y z",
         "(print",
         "(print)",
         "(print (",
         "(print x",
         "(print x (",
         "(print x y",
         "(print x y)",
         "(apply",
         "(apply)",
         "(apply (",
         "(apply f",
         "(apply f (",
         "(apply f x",
         "(apply f x (",
         "(apply f x y",
         // test lambda
         "(prog\n(fun f\n(",
         "(prog\n(fun f\n(var x 0)",
         "(prog\n(fun f\n(lambda",
         "(prog\n(fun f\n(lambda)",
         "(prog\n(fun f\n(lambda 0 x)",
         "(prog\n(fun f\n(lambda x)",
         "(prog\n(fun f\n(lambda (",
         "(prog\n(fun f\n(lambda x",
         "(prog\n(fun f\n(lambda x (",
         "(prog\n(fun f\n(lambda x x",
         // test fun
         "(prog\n(fun",
         "(prog\n(fun)",
         "(prog\n(fun (",
         "(prog\n(fun 0",
         "(prog\n(fun var",
         "(prog\n(fun lambda",
         "(prog\n(fun f)",
         "(prog\n(fun (lambda 0)",
         "(prog\n(fun f\n(lambda 0)",
         "(prog\n(fun f x\n(lambda 0)",
         // test arithmetic and boolean expressions
         "(+",
         "(-",
         "(*",
         "(/",
         "(%",
         "(^",
         "(!",
         "(||",
         "(&&",
         "(==",
         "(!=",
         "(<=",
         "(>=",
         "(<",
         "(>",
         "(* 1",
         "(/ 1",
         "(% 1",
         "(^ 1)",
         "(* 1)",
         "(/ 1)",
         "(% 1)",
         "(^ 1)",
         "(|| false",
         "(&& false",
         "(|| false)",
         "(&& false)",
         "(== 1)",
         "(!= 1)",
         "(<= 1)",
         "(>= 1)",
         "(<  1)",
         "(>  1)",
         "(! true false)",
         "(- 1 2 3)",
         "(/ 1 2 3)",
         "(% 1 2 3)",
         "(^ 1 2 3)",
         "(== 1 2 3",
         "(!= 1 2 3",
         "(<= 1 2 3",
         "(>= 1 2 3",
         "(<  1 2 3",
         "(>  1 2 3",
         "(|| false false false",
         "(&& false false false",
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         System.out.println( programs[i] + "\n" );

         // Build and evaluate the AST that represents the expression.
         try
         {
            Tokenizer tokens = new Tokenizer(programs[i]);

            // Print the token list.
            //System.out.println( tokens + "\n" );

            Tree ast = Parse2AST.parse(tokens);
            //Tree ast = BuildTree.buildTree(tokens);
            System.out.println("***** WARNING: This expression should have caused a parsing error!\n");
            System.exit(0);
         }
         catch (TokenizeException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()
}
